/* rand.c */

/*
 * Copyright (c) 1997-2002 Hoshi Takanori
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <PalmOS.h>

#ifdef USE_HANDERA_VGA
#include <Vga.h>
#endif

#include "life.h"
#include "life_rcp.h"


/*
 * generating random life pattern
 */

static offset_t random_rect(
    cell_t *tape, offset_t tape_length,
    coord_t width, coord_t height, int density)
{
    coord_t x, y;
    offset_t max, length = 0;

    max = tape_length / 3 - 1;
    if (width <= 0 || height <= 0 || density <= 0
     || (long) width * height * density / 80 >= max) {
	return -1;
    }

    for (y = 0; y < height; ++y) {
	for (x = 0; x < width; ++x) {
	    if (SysRandom(0) < ((long) sysRandomMax + 1) * density / 100) {
		if (length < max) {
		    tape[length++] = CELL(x - width / 2, y - height / 2);
		}
	    }
	}
    }

    return length;
}

static offset_t random_circle(
    cell_t *tape, offset_t tape_length,
    coord_t radius, int density1, int density2)
{
    coord_t x, y, r = radius;
    long square, density;
    offset_t max, length = 0;

    max = tape_length / 3 - 1;
    if (radius <= 0
     || density1 < 0 || density2 < 0 || (density1 + density2 <= 0)
     || (long) radius * radius * (density1 + density2 * 2) / 80 >= max) {
	return -1;
    }

    for (y = - radius; y <= radius; ++y) {
	for (x = - radius; x <= radius; ++x) {
	    square = (long) x * x + (long) y * y;
	    if (square <= (long) radius * radius) {
		if ((long) r * r <= square) {
		    do {
			++r;
		    } while ((long) r * r <= square);
		    --r;
		} else {
		    do {
			--r;
		    } while ((long) r * r > square);
		}

		density = (long) density1 * (radius - r) + (long) density2 * r;
		if ((long) SysRandom(0) * radius
			< ((long) sysRandomMax + 1) * density / 100) {
		    if (length < max) {
			tape[length++] = CELL(x, y);
		    }
		}
	    }
	}
    }

    return length;
}

static void show_hide_rect_fields(FormPtr dialog, Boolean show)
{
    ShowHideObject(dialog, random_width_label_id, show);
    ShowHideObject(dialog, random_width_field_id, show);
    ShowHideObject(dialog, random_height_label_id, show);
    ShowHideObject(dialog, random_height_field_id, show);
    ShowHideObject(dialog, random_density_label_id, show);
    ShowHideObject(dialog, random_density_field_id, show);
}

static void show_hide_circle_fields(FormPtr dialog, Boolean show)
{
    ShowHideObject(dialog, random_radius_label_id, show);
    ShowHideObject(dialog, random_radius_field_id, show);
    ShowHideObject(dialog, random_density1_label_id, show);
    ShowHideObject(dialog, random_density1_field_id, show);
    ShowHideObject(dialog, random_density2_label_id, show);
    ShowHideObject(dialog, random_density2_field_id, show);
}

static Boolean handle_random_dialog_event(EventPtr event)
{
    Boolean handled = false;

    switch (event->eType) {
    case ctlSelectEvent:
	switch (event->data.ctlSelect.controlID) {
	case random_rect_id:
	    show_hide_circle_fields(FrmGetActiveForm(), false);
	    show_hide_rect_fields(FrmGetActiveForm(), true);
	    handled = true;
	    break;

	case random_circle_id:
	    show_hide_rect_fields(FrmGetActiveForm(), false);
	    show_hide_circle_fields(FrmGetActiveForm(), true);
	    handled = true;
	    break;
	}
	break;

    default:
	break;
    }

    return handled;
}

void random_life(cell_t *tape, offset_t tape_length)
{
    FormPtr dialog;
    int button;
    offset_t length;

    dialog = FrmInitForm(random_form_id);
#ifdef USE_HANDERA_VGA
    if (handera_vga) {
	VgaFormModify(dialog, vgaFormModify160To240);
    }
#endif
    if (prefs.rand_type == 0) {
	FrmSetControlGroupSelection(dialog, 1, random_rect_id);
	show_hide_rect_fields(dialog, true);
    } else {
	FrmSetControlGroupSelection(dialog, 1, random_circle_id);
	show_hide_circle_fields(dialog, true);
    }
    SetFieldNumHandle(dialog, random_width_field_id, prefs.rand_width);
    SetFieldNumHandle(dialog, random_height_field_id, prefs.rand_height);
    SetFieldNumHandle(dialog, random_density_field_id, prefs.rand_density);
    SetFieldNumHandle(dialog, random_radius_field_id, prefs.rand_radius);
    SetFieldNumHandle(dialog, random_density1_field_id, prefs.rand_density1);
    SetFieldNumHandle(dialog, random_density2_field_id, prefs.rand_density2);
    FrmSetEventHandler(dialog, handle_random_dialog_event);

    button = FrmDoDialog(dialog);
    if (button == random_ok_id) {
	if (CtlGetValue(GetObjectPtrIn(dialog, random_rect_id))) {
	    prefs.rand_type = 0;
	} else {
	    prefs.rand_type = 1;
	}
	prefs.rand_width = GetFieldNumber(dialog, random_width_field_id);
	prefs.rand_height = GetFieldNumber(dialog, random_height_field_id);
	prefs.rand_density = GetFieldNumber(dialog, random_density_field_id);
	prefs.rand_radius = GetFieldNumber(dialog, random_radius_field_id);
	prefs.rand_density1 = GetFieldNumber(dialog, random_density1_field_id);
	prefs.rand_density2 = GetFieldNumber(dialog, random_density2_field_id);
    }
    FrmDeleteForm(dialog);

    if (button == random_ok_id) {
	if (prefs.rand_type == 0) {
	    length = random_rect(tape, tape_length,
		prefs.rand_width, prefs.rand_height, prefs.rand_density);
	} else {
	    length = random_circle(tape, tape_length,
		prefs.rand_radius, prefs.rand_density1, prefs.rand_density2);
	}

	if (length < 0) {
	    FrmAlert(random_error_id);
	} else {
	    reset_life(length);
	}
    }
}
