/* util.c */

/*
 * Copyright (c) 1997-2002 Hoshi Takanori
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <PalmOS.h>

#include "life.h"


/*
 * common routines
 */

void *GetObjectPtrIn(FormPtr form, UInt16 id)
{
    return FrmGetObjectPtr(form, FrmGetObjectIndex(form, id));
}

void *GetObjectPtr(UInt16 id)
{
    return GetObjectPtrIn(FrmGetActiveForm(), id);
}

void GetObjectBounds(UInt16 id, RectanglePtr bounds)
{
    FormPtr form = FrmGetActiveForm();

    FrmGetObjectBounds(form, FrmGetObjectIndex(form, id), bounds);
}

void ShowHideObject(FormPtr form, UInt16 id, Boolean show)
{
    if (show) {
	FrmShowObject(form, FrmGetObjectIndex(form, id));
    } else {
	FrmHideObject(form, FrmGetObjectIndex(form, id));
    }
}

void SetFieldNumber(FieldPtr field, Char *str, long number)
{
    StrIToA(str, number);
    FldSetTextPtr(field, str);
    FldDrawField(field);
}

void SetFieldNumHandle(FormPtr form, UInt16 id, long number)
{
    FieldPtr field;
    MemHandle h;
    Char *str;

    field = GetObjectPtrIn(form, id);

    h = MemHandleNew(12);
    if (h != NULL) {
	str = MemHandleLock(h);
	StrIToA(str, number);
	MemHandleUnlock(h);
	FldFreeMemory(field);
	FldSetTextHandle(field, h);
    }
}

long GetFieldNumber(FormPtr form, UInt16 id)
{
    FieldPtr field;
    const Char *str;

    field = GetObjectPtrIn(form, id);
    str = FldGetTextPtr(field);
    if (str != NULL && *str != '\0') {
	return StrAToI(str);
    } else {
	return 0;
    }
}

UInt16 FindCategory(DmOpenRef db_ref, const Char *category_name)
{
    char name[dmCategoryLength];
    UInt16 i;

    for (i = 0; i < dmRecNumCategories; ++i) {
	CategoryGetName(db_ref, i, name);
	if (StrCaselessCompare(name, category_name) == 0) {
	    break;
	}
    }
    return i;
}

Boolean IsMenuVisible(void)
{
    MenuBarType *mbar = MenuGetActiveMenu();

    return (mbar != NULL && mbar->attr.visible);
}

Int16 GetTopItem(ListPtr list)
{
    if (rom_version >= sysMakeROMVersion(4, 0, 0, 0, 0)) {
	return LstGetTopItem(list);
    } else {
	return list->topItem;
    }
}
